/**
 * 
 */
package org.processmining.models.graphbased.directed.aapdgraph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import javax.swing.JLabel;
import javax.swing.SwingUtilities;

import org.processmining.models.aggregatedactivitiesperformancediagram.AAPDElement;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.shapes.AAPDBarShape;
import org.processmining.models.shapes.Decorated;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version May 18, 2009
 */
public abstract class AAPDBar extends AbstractDirectedGraphNode implements Decorated {
	// linkage to graph
	private final AbstractDirectedGraph<AAPDBar, AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>> graph;

	private final static int CELLPADDING = 14;
	private final static int FOCUSELEMENTCELLPADDING = 5;

	// information about decoration location
	private boolean isFocusElement;
	private AAPDElement aapdFocusElements;
	private double posXStart;
	private double posYStart;
	private double waitingTime;
	private double serviceTime;
	private double realThroughputTime;
	private double intersectionTime;
	private double heightOfElements;
	private double lineHeight;
	private double intersectionBarHeight;

	// boundary value
	private double boundaryLow;
	private double boundaryHigh;

	// other information
	/*
	 * [HV] The field AAPDBar.frequencyActivityInstances is never read locally
	 * private int frequencyActivityInstances = 0;
	 */
	private double multiplier = 1.0;

	public AAPDBar(AbstractDirectedGraph<AAPDBar, AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>> graph,
			AAPDElement aapdFocusElements, double posXStart, double posYStart, double waitingTime, double serviceTime,
			double realThroughputTime, double intersectionTime, double heightOfElements, double lineHeight,
			double intersectionBarHeight, double boundaryLow, double boundaryHigh, boolean isFocusElement) {
		super();
		this.graph = graph;

		// internal data
		this.isFocusElement = isFocusElement;
		this.aapdFocusElements = aapdFocusElements;
		this.posXStart = posXStart;
		this.posYStart = posYStart;
		this.waitingTime = waitingTime > 0 ? waitingTime : 1; // this is needed for visualization
		this.serviceTime = serviceTime > 0 ? serviceTime : 1;
		this.realThroughputTime = realThroughputTime;
		this.intersectionTime = intersectionTime;
		this.heightOfElements = heightOfElements;
		this.lineHeight = lineHeight;
		this.intersectionBarHeight = intersectionBarHeight;

		// set boundary
		this.boundaryLow = boundaryLow;
		this.boundaryHigh = boundaryHigh;

		getAttributeMap().put(AttributeMap.SHAPE, new AAPDBarShape());
		getAttributeMap().put(AttributeMap.RESIZABLE, false);

		// size depends
		JLabel label = new JLabel(aapdFocusElements.getLabel());
		FontMetrics fm = label.getFontMetrics(new Font("Arial", Font.PLAIN, 12));
		int fontLength = SwingUtilities.computeStringWidth(fm, aapdFocusElements.getLabel());

		double throughputTime = (this.serviceTime + this.waitingTime);
		if ((fontLength + CELLPADDING + CELLPADDING) > throughputTime) {
			if (!isFocusElement) {
				// draw string on the right side of the bar
				getAttributeMap().put(
						AttributeMap.SIZE,
						new Dimension((int) (this.waitingTime + this.serviceTime) + fontLength + CELLPADDING
								+ CELLPADDING, (int) (this.heightOfElements + this.intersectionBarHeight))); // default value
			} else {
				// draw string on the right side of the bar
				getAttributeMap().put(
						AttributeMap.SIZE,
						new Dimension((int) (this.waitingTime + this.serviceTime) + fontLength + (2 * CELLPADDING)
								+ (2 * FOCUSELEMENTCELLPADDING),
								(int) (this.heightOfElements + this.intersectionBarHeight)
										+ (2 * FOCUSELEMENTCELLPADDING))); // default value
			}
		} else {
			if (!isFocusElement) {
				// draw string inside of the bar
				getAttributeMap().put(
						AttributeMap.SIZE,
						new Dimension((int) (this.waitingTime + this.serviceTime),
								(int) (this.heightOfElements + this.intersectionBarHeight))); // default value
			} else {
				// draw string inside of the bar
				getAttributeMap().put(
						AttributeMap.SIZE,
						new Dimension((int) (this.waitingTime + this.serviceTime) + (2 * FOCUSELEMENTCELLPADDING),
								(int) (this.heightOfElements + this.intersectionBarHeight)
										+ (2 * FOCUSELEMENTCELLPADDING))); // default value
			}
		}

		getAttributeMap().put(AttributeMap.LABEL, aapdFocusElements.getLabel());
		getAttributeMap().put(AttributeMap.SHOWLABEL, false);
		getAttributeMap().put(AttributeMap.MOVEABLE, false);
		getAttributeMap().put(AttributeMap.STROKECOLOR, Color.WHITE);
	}

	public AbstractDirectedGraph<AAPDBar, AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>> getGraph() {
		return graph;
	}

	/**
	 * @return the boundaryLow
	 */
	public double getBoundaryLow() {
		return boundaryLow;
	}

	/**
	 * @param boundaryLow
	 *            the boundaryLow to set
	 */
	public void setBoundaryLow(double boundaryLow) {
		this.boundaryLow = boundaryLow;
	}

	/**
	 * @return the boundaryHigh
	 */
	public double getBoundaryHigh() {
		return boundaryHigh;
	}

	/**
	 * @param boundaryHigh
	 *            the boundaryHigh to set
	 */
	public void setBoundaryHigh(double boundaryHigh) {
		this.boundaryHigh = boundaryHigh;
	}

	/**
	 * @return the aapdFocusElements
	 */
	public AAPDElement getAapdFocusElements() {
		return aapdFocusElements;
	}

	/**
	 * @param aapdFocusElements
	 *            the aapdFocusElements to set
	 */
	public void setAapdFocusElements(AAPDElement aapdFocusElements) {
		this.aapdFocusElements = aapdFocusElements;
	}

	/**
	 * @return the posXStart
	 */
	public double getPosXStart() {
		return posXStart;
	}

	/**
	 * @param posXStart
	 *            the posXStart to set
	 */
	public void setPosXStart(double posXStart) {
		this.posXStart = posXStart;
	}

	/**
	 * @return the posYStart
	 */
	public double getPosYStart() {
		return posYStart;
	}

	/**
	 * @param posYStart
	 *            the posYStart to set
	 */
	public void setPosYStart(double posYStart) {
		this.posYStart = posYStart;
	}

	/**
	 * @return the waitingTime
	 */
	public double getWaitingTime() {
		return waitingTime;
	}

	/**
	 * @param waitingTime
	 *            the waitingTime to set
	 */
	public void setWaitingTime(double waitingTime) {
		this.waitingTime = waitingTime;
	}

	/**
	 * @return the serviceTime
	 */
	public double getServiceTime() {
		return serviceTime;
	}

	/**
	 * @param serviceTime
	 *            the serviceTime to set
	 */
	public void setServiceTime(double serviceTime) {
		this.serviceTime = serviceTime;
	}

	/**
	 * @return the intersectionTime
	 */
	public double getIntersectionTime() {
		return intersectionTime;
	}

	/**
	 * @param intersectionTime
	 *            the intersectionTime to set
	 */
	public void setIntersectionTime(double intersectionTime) {
		this.intersectionTime = intersectionTime;
	}

	/**
	 * @return the heightOfElements
	 */
	public double getHeightOfElements() {
		return heightOfElements;
	}

	/**
	 * @param heightOfElements
	 *            the heightOfElements to set
	 */
	public void setHeightOfElements(double heightOfElements) {
		this.heightOfElements = heightOfElements;
	}

	/**
	 * @return the lineHeight
	 */
	public double getLineHeight() {
		return lineHeight;
	}

	/**
	 * @param lineHeight
	 *            the lineHeight to set
	 */
	public void setLineHeight(double lineHeight) {
		this.lineHeight = lineHeight;
	}

	/**
	 * @return the multiplier
	 */
	public double getMultiplier() {
		return multiplier;
	}

	/**
	 * @param multiplier
	 *            the multiplier to set
	 */
	public void setMultiplier(double multiplier) {
		this.multiplier = multiplier;
	}

	/**
	 * GUI related methods
	 */

	public void decorate(Graphics2D g2d, double x, double y, double width, double height) {
		GeneralPath path = new GeneralPath();

		double x_addition = 0;
		double y_addition = 0;

		// general stroke
		BasicStroke stroke = new BasicStroke((float) 1.0);
		g2d.setStroke(stroke);

		if (isFocusElement) {
			// draw red marker
			g2d.setColor(Color.PINK);
			path.append(new Rectangle2D.Double(x, y, waitingTime + serviceTime + (2 * FOCUSELEMENTCELLPADDING),
					heightOfElements + (2 * FOCUSELEMENTCELLPADDING) + intersectionBarHeight), false);
			g2d.fill(path);

			x_addition = FOCUSELEMENTCELLPADDING;
			y_addition = FOCUSELEMENTCELLPADDING;
		}

		// draw fillings for waiting time
		path.reset();
		g2d.setColor(Color.LIGHT_GRAY);
		path.append(new Rectangle2D.Double(x + x_addition, y + y_addition, waitingTime, heightOfElements), false);
		g2d.fill(path);

		// draw fillings for service time
		path.reset();
		path.append(
				new Rectangle2D.Double(x + waitingTime + x_addition, y + y_addition, serviceTime, heightOfElements),
				false);
		g2d.setColor(getPerformanceColor(getRealThroughputTime(), getBoundaryLow(), getBoundaryHigh()));
		g2d.fill(path);

		// fill rest of intersection time with black
		path.reset();
		path.append(new Rectangle2D.Double(x + x_addition, y + heightOfElements + y_addition,
				(waitingTime + serviceTime), intersectionBarHeight), false);
		g2d.setColor(Color.BLACK);
		g2d.fill(path);

		// draw intersection time
		path.reset();
		path.append(new Rectangle2D.Double(x + x_addition, y + heightOfElements + y_addition, intersectionTime,
				intersectionBarHeight), false);
		g2d.setColor(Color.CYAN);
		g2d.fill(path);

		// draw line
		path.reset();
		double middleBar = heightOfElements / 2;
		g2d.setColor(Color.BLUE);
		path.moveTo(new Float(x + waitingTime + x_addition),
				(float) (y + middleBar - (middleBar * lineHeight) + y_addition));
		path.lineTo(new Float(x + waitingTime + serviceTime + x_addition), new Float(y + middleBar
				- (middleBar * lineHeight) + y_addition));
		path.closePath();
		g2d.draw(path);

		// draw label
		g2d.setColor(Color.BLACK);
		//		FontMetrics fm = g2d.getFontMetrics();
		//		int fontLength = SwingUtilities.computeStringWidth(fm, aapdFocusElements.getLabel());

		// BVD: Added this code in order to correctly render HTML. 
		//
		int labelX;
		int labelY;

		JLabel label = new JLabel(aapdFocusElements.getLabel());
		label.validate();
		Dimension d = label.getPreferredSize();
		label.setSize(d);

		double throughputTime = (serviceTime + waitingTime);
		if ((d.getWidth() + CELLPADDING + CELLPADDING) > throughputTime) {
			// BVD: REMOVED FOR HTML RENDERING 
			// draw string on the right side of the bar
			//						g2d.drawString(aapdFocusElements.getLabel(), (int) (x + throughputTime + CELLPADDING + x_addition),
			//								(int) (y + middleBar + y_addition));

			// The left center point of the label should be located at 
			// - (int)(x + throughputTime + CELLPADDING + x_addition),(int)(y + middleBar + y_addition)
			labelX = (int) (x + throughputTime + CELLPADDING + x_addition);
			labelY = (int) (y + middleBar + y_addition);
		} else {
			// BVD: REMOVED FOR HTML RENDERING 
			// draw string inside of the bar
			//						g2d.drawString(aapdFocusElements.getLabel(),
			//								(int) (x + (throughputTime / 2) - (fontLength / 2) + x_addition),
			//								(int) (y + middleBar + y_addition));

			// The left center point of the label should be located at 
			// - (int)(x + throughputTime + CELLPADDING + x_addition),(int)(y + middleBar + y_addition)
			labelX = (int) (x + (throughputTime / 2) - (d.getWidth() / 2) + x_addition);
			labelY = (int) (y + middleBar + y_addition);
		}

		label.paint(g2d.create(labelX, (int) (labelY - d.getHeight() / 2), (int) d.getWidth(), (int) d.getHeight()));

	}

	private Color getPerformanceColor(double time, double timeLow, double timeHigh) {
		if (time <= timeLow) {
			return Color.GREEN;
		} else if (time <= timeHigh) {
			return Color.YELLOW;
		} else {
			return Color.RED;
		}
	}

	public double getIntersectionBarHeight() {
		return intersectionBarHeight;
	}

	public void setIntersectionBarHeight(double intersectionBarHeight) {
		this.intersectionBarHeight = intersectionBarHeight;
	}

	/**
	 * @return the isFocusElement
	 */
	public boolean isFocusElement() {
		return isFocusElement;
	}

	/**
	 * @param isFocusElement
	 *            the isFocusElement to set
	 */
	public void setFocusElement(boolean isFocusElement) {
		this.isFocusElement = isFocusElement;
	}

	/**
	 * @return the realThroughputTime
	 */
	public double getRealThroughputTime() {
		return realThroughputTime;
	}

	/**
	 * @param realThroughputTime
	 *            the realThroughputTime to set
	 */
	public void setRealThroughputTime(double realThroughputTime) {
		this.realThroughputTime = realThroughputTime;
	}

}
