/**
 * 
 */
package org.processmining.models.graphbased.directed.aapdgraph;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.SwingConstants;

import org.processmining.models.aggregatedactivitiesperformancediagram.AAPDElement;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.aapdgraph.elements.AAPDBarEdgeElement;
import org.processmining.models.graphbased.directed.aapdgraph.elements.AAPDBarElement;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version May 18, 2009
 */
public class AAPDGraphImpl extends AbstractDirectedGraph<AAPDBar, AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>>
		implements AAPDGraph {

	protected final Set<AAPDBar> aapdBars;
	protected final Set<AAPDBarEdge<AAPDBar, AAPDBar>> arcs;
	protected AAPDBar focusElements;

	public AAPDGraphImpl(String label) {
		super();
		aapdBars = new LinkedHashSet<AAPDBar>();
		arcs = new LinkedHashSet<AAPDBarEdge<AAPDBar, AAPDBar>>();

		getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.WEST);
		getAttributeMap().put(AttributeMap.LABEL, label);
	}

	protected Map<? extends DirectedGraphElement, ? extends DirectedGraphElement> cloneFrom(
			DirectedGraph<AAPDBar, AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>> graph) {
		HashMap<DirectedGraphElement, DirectedGraphElement> mapping = new HashMap<DirectedGraphElement, DirectedGraphElement>();

		for (AAPDBar bar : graph.getNodes()) {
			mapping.put(bar, addNode(bar.getAapdFocusElements(), bar.getPosXStart(), bar.getPosYStart(), bar
					.getWaitingTime(), bar.getServiceTime(), bar.getRealThroughputTime(), bar.getIntersectionTime(),
					bar.getHeightOfElements(), bar.getLineHeight(), bar.getIntersectionBarHeight(), bar
							.getBoundaryLow(), bar.getBoundaryHigh(), bar.isFocusElement()));
		}
		getAttributeMap().clear();
		AttributeMap map = graph.getAttributeMap();
		for (String key : map.keySet()) {
			getAttributeMap().put(key, map.get(key));
		}
		return mapping;
	}

	@Override
	protected AAPDGraphImpl getEmptyClone() {
		return new AAPDGraphImpl(getLabel());
	}

	/**
	 * BARS (node)
	 */
	public synchronized AAPDBar addNode(AAPDElement aapdFocusElements, double posXStart, double posYStart,
			double waitingTime, double serviceTime, double realThroughputTime, double intersectionTime,
			double heightOfElements, double lineHeight, double intersectionBarHeight, double boundaryLow,
			double boundaryHigh, boolean isFocusElement) {
		AAPDBar aapdBar = new AAPDBarElement(this, aapdFocusElements, posXStart, posYStart, waitingTime, serviceTime,
				realThroughputTime, intersectionTime, heightOfElements, lineHeight, intersectionBarHeight, boundaryLow,
				boundaryHigh, isFocusElement);
		aapdBars.add(aapdBar);
		graphElementAdded(aapdBar);
		return aapdBar;
	}

	public synchronized AAPDBar removeNode(AAPDBar aapdBar) {
		return removeNodeFromCollection(aapdBars, aapdBar);
	}

	public void removeNode(DirectedGraphNode cell) {
		if (cell instanceof AAPDBar) {
			removeNode((AAPDBar) cell);
		}
	}

	public synchronized Set<AAPDBar> getNodes() {
		Set<AAPDBar> newNodes = new HashSet<AAPDBar>();
		newNodes.addAll(aapdBars);
		return newNodes;
	}

	/**
	 * DUMMY ARC (arcs are never added)
	 */
	public synchronized AAPDBarEdgeElement addArc(AAPDBar source, AAPDBar target) {
		return addArc(source, target, null);
	}

	public synchronized AAPDBarEdgeElement addArc(AAPDBar source, AAPDBar target, String label) {
		return null;
	}

	public synchronized AAPDBarEdge<AAPDBar, AAPDBar> removeArc(AAPDBar source, AAPDBar target) {
		return removeFromEdges(source, target, arcs);
	}

	public AAPDBarEdge<AAPDBar, AAPDBar> removeArc(AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar> edge) {
		return removeFromEdges(edge.getSource(), edge.getTarget(), arcs);
	}

	@SuppressWarnings("unchecked")
	public void removeEdge(DirectedGraphEdge edge) {
		if (edge instanceof AAPDBarEdgeElement) {
			arcs.remove(edge);
		} else {
			assert (false);
		}
		graphElementRemoved(edge);
	}

	public synchronized AAPDBarEdge<AAPDBar, AAPDBar> getArc(AAPDBar source, AAPDBar target) {
		Collection<AAPDBarEdge<AAPDBar, AAPDBar>> set = getEdges(source, target, arcs);
		return (set.isEmpty() ? null : set.iterator().next());
	}

	public synchronized Set<AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>> getEdges() {
		Set<AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>> edges = new HashSet<AAPDBarEdge<? extends AAPDBar, ? extends AAPDBar>>();
		edges.addAll(arcs);
		return edges;
	}
}
