/**
 * 
 */
package org.processmining.models.performancemeasurement.dataelements;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.framework.util.Pair;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Apr 5, 2009
 */
public class ActivityInstanceBar {
	public static final int NOLIFECYCLETRANSITION = -2;

	private int caseID; // case number where the ActivityInstance exist 
	private int barID; // ID for this bar
	private XEventClass eventClass; // store an event class which corresponds to this node
	private List<Pair<Integer, Date>> listEventTime = new LinkedList<Pair<Integer, Date>>(); // store list of lifecycle transition and its time occurence

	@SuppressWarnings("unused")
	private ActivityInstanceBar() {
	}

	/**
	 * Constructor
	 * 
	 * @param eventClass
	 * @param caseID
	 */
	public ActivityInstanceBar(XEventClass eventClass, int caseID, int barID) {
		this.eventClass = eventClass;
		this.caseID = caseID;
		this.barID = barID;
	}

	/**
	 * @return the caseID
	 */
	public int getCaseID() {
		return caseID;
	}

	/**
	 * @param caseID
	 *            the caseID to set
	 */
	public void setCaseID(int caseID) {
		this.caseID = caseID;
	}

	/**
	 * @return the barID
	 */
	public int getBarID() {
		return barID;
	}

	/**
	 * @param barID
	 *            the barID to set
	 */
	public void setBarID(int barID) {
		this.barID = barID;
	}

	/**
	 * @return the eventClass
	 */
	public XEventClass getEventClass() {
		return eventClass;
	}

	/**
	 * @param eventClass
	 *            the eventClass to set
	 */
	public void setEventClass(XEventClass eventClass) {
		this.eventClass = eventClass;
	}

	/**
	 * @return the listEventTime
	 */
	public List<Pair<Integer, Date>> getListEventTime() {
		return listEventTime;
	}

	/**
	 * @param listEventTime
	 *            the listEventTime to set
	 */
	public void setListEventTime(List<Pair<Integer, Date>> listEventTime) {
		this.listEventTime = listEventTime;
	}

	/**
	 * Add a new lifecycle transition to this activity instance bar
	 * 
	 * @param newLifecycleTransition
	 *            observed lifecycle transition, can be null
	 * @param timeoccurence
	 */
	public void addLifecycleTransition(int newLifecycleTransition, Date timeoccurence) {
		listEventTime.add(new Pair<Integer, Date>(newLifecycleTransition, timeoccurence));
	}

	/**
	 * remove an existing lifecycle transition from this activity instance bar
	 * 
	 * @param removedLifecycleTransition
	 * @param timeoccurence
	 */
	public void removeLifecycleTransition(int removedLifecycleTransition, Date timeoccurence) {
		int counter = 0;
		while (counter < listEventTime.size()) {
			if (removedLifecycleTransition == listEventTime.get(counter).getFirst()) {
				if (timeoccurence != null) {
					if (timeoccurence.equals(listEventTime.get(counter).getSecond())) {
						listEventTime.remove(counter);
						break;
					}
					;
				} else {
					if (listEventTime.get(counter).getSecond() == null) {
						listEventTime.remove(counter);
						break;
					}
				}
			}
			;
		}
	}

	/**
	 * Get start time of this activity instance
	 * 
	 * @return
	 */
	public Date getStartTime() {
		int counter = 0;
		while (counter < listEventTime.size()) {
			if (listEventTime.get(counter).getSecond() != null) {
				return listEventTime.get(counter).getSecond();
			}
			counter++;
		}
		return null;
	}

	/**
	 * Get end time of this activity instance
	 * 
	 * @return
	 */
	public Date getEndTime() {
		int counter = listEventTime.size() - 1;
		while (counter >= 0) {
			if (listEventTime.get(counter).getSecond() != null) {
				return listEventTime.get(counter).getSecond();
			}
			counter--;
		}
		return null;
	}

	/**
	 * Get duration of this activity instance
	 * 
	 * @return -1 if there is no event which has time info in this activity
	 *         instance
	 */
	public long getDuration() {
		Date startTime = getStartTime();
		Date endTime = getEndTime();
		if ((startTime != null) && (endTime != null)) {
			return endTime.getTime() - startTime.getTime();
		} else {
			return -1;
		}
	}

	/**
	 * get the latest lifecycle transition for this activity instance
	 * 
	 * @return
	 */
	public int getLatestLifecycleTransition() {
		if (listEventTime.size() > 0) {
			return (listEventTime.get(listEventTime.size() - 1).getFirst());
		}
		return NOLIFECYCLETRANSITION;
	}

}
