/**
 * 
 */
package org.processmining.models.performancemeasurement.dataelements;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 19, 2009
 */
public class CaseKPIData {
	// general measurements
	private int numCases = 0;
	private double arrivalRate = 0.0;
	private int numInvolvedResource = 0;
	private int numInvolvedTeams = 0;
	private int numUniqueTraces = 0;
	private double executedEventsPerResource = 0.0;
	private double executedActivityPerResource = 0.0;
	private double resourcePerCase = 0.0;

	private Set<Integer> fittingCaseIDs;

	// TABLE STATISTIC
	private List<Long> caseThroughputTimeData; // all case throughput time is stored here

	/**
	 * Default constructor
	 */
	public CaseKPIData() {
		caseThroughputTimeData = new LinkedList<Long>();
		fittingCaseIDs = new HashSet<Integer>();
	}

	/**
	 * add caseID in which no violating paths are encountered
	 * 
	 * @param caseID
	 */
	public void addFittingCaseID(int caseID) {
		fittingCaseIDs.add(caseID);
	}

	/**
	 * remove caseID from fitting case ID record
	 * 
	 * @param caseID
	 */
	public void removeFittingCaseID(int caseID) {
		fittingCaseIDs.remove(caseID);
	}

	/**
	 * clear fitting case ID
	 */
	public void clearFittingCaseID() {
		fittingCaseIDs.clear();
	}

	/**
	 * get number of cases which fits FPD
	 * 
	 * @return
	 */
	public int getNumFittingCase() {
		return fittingCaseIDs.size();
	}

	/**
	 * get set of case IDs which fit FPD
	 * 
	 * @return
	 */
	public Set<Integer> getFittingCaseID() {
		return fittingCaseIDs;
	}

	/**
	 * set case IDs which fit FPD
	 * 
	 * @param setFittingCaseID
	 */
	public void setFittingCaseID(Set<Integer> setFittingCaseID) {
		fittingCaseIDs = setFittingCaseID;
	}

	/**
	 * get number of cases
	 * 
	 * @return
	 */
	public int getNumCases() {
		return numCases;
	}

	/**
	 * set number of cases
	 * 
	 * @param numCases
	 */
	public void setNumCases(int numCases) {
		this.numCases = numCases;
	}

	/**
	 * get arrival rate of cases
	 * 
	 * @return
	 */
	public double getArrivalRate() {
		return arrivalRate;
	}

	/**
	 * set arrival rate of cases
	 * 
	 * @param arrivalRate
	 */
	public void setArrivalRate(double arrivalRate) {
		this.arrivalRate = arrivalRate;
	}

	/**
	 * get number of resources who are involved
	 * 
	 * @return
	 */
	public int getNumInvolvedResource() {
		return numInvolvedResource;
	}

	/**
	 * set number of resources who are involved
	 * 
	 * @param numInvolvedResource
	 */
	public void setNumInvolvedResource(int numInvolvedResource) {
		this.numInvolvedResource = numInvolvedResource;
	}

	/**
	 * get number of unique traces
	 * 
	 * @return
	 */
	public int getNumUniqueTraces() {
		return numUniqueTraces;
	}

	/**
	 * set number of unique traces
	 * 
	 * @param numUniqueTraces
	 */
	public void setNumUniqueTraces(int numUniqueTraces) {
		this.numUniqueTraces = numUniqueTraces;
	}

	/**
	 * get number of events which are executed per resource
	 * 
	 * @return
	 */
	public double getExecutedEventsPerResource() {
		return executedEventsPerResource;
	}

	/**
	 * set number of events which are executed per resource
	 * 
	 * @param executedEventsPerResource
	 */
	public void setExecutedEventsPerResource(double executedEventsPerResource) {
		this.executedEventsPerResource = executedEventsPerResource;
	}

	/**
	 * get number of executed activity per resource
	 * 
	 * @return
	 */
	public double getExecutedActivityPerResource() {
		return executedActivityPerResource;
	}

	/**
	 * set number of executed activity per resource
	 * 
	 * @param executedActivityPerResource
	 */
	public void setExecutedActivityPerResource(double executedActivityPerResource) {
		this.executedActivityPerResource = executedActivityPerResource;
	}

	/**
	 * get average number of resource which are involved per case
	 * 
	 * @return
	 */
	public double getResourcePerCase() {
		return resourcePerCase;
	}

	/**
	 * set average number of resource which are involved per case
	 * 
	 * @param resourcePerCase
	 */
	public void setResourcePerCase(double resourcePerCase) {
		this.resourcePerCase = resourcePerCase;
	}

	/**
	 * @return the caseThroughputTimeData
	 */
	public List<Long> getCaseThroughputTimeData() {
		return caseThroughputTimeData;
	}

	/**
	 * @param caseThroughputTimeData
	 *            the caseThroughputTimeData to set
	 */
	public void setCaseThroughputTimeData(List<Long> caseThroughputTimeData) {
		this.caseThroughputTimeData = caseThroughputTimeData;
	}

	/**
	 * remove all cases throughput time data
	 */
	public void removeAllCaseThroughputTimeData() {
		caseThroughputTimeData.clear();
	}

	/**
	 * Add all case throughput time data
	 * 
	 * @param collection
	 */
	public void addAllCaseThroughputTimeData(Collection<Long> collection) {
		caseThroughputTimeData.addAll(collection);
	}

	/**
	 * add case throughput time data
	 * 
	 * @param newThroughputTime
	 */
	public void addCaseThroughputTimeData(Long newThroughputTime) {
		caseThroughputTimeData.add(newThroughputTime);
	}

	/**
	 * @return the numInvolvedTeams
	 */
	public int getNumInvolvedTeams() {
		return numInvolvedTeams;
	}

	/**
	 * @param numInvolvedTeams
	 *            the numInvolvedTeams to set
	 */
	public void setNumInvolvedTeams(int numInvolvedTeams) {
		this.numInvolvedTeams = numInvolvedTeams;
	}
}
