/**
 * 
 */
package org.processmining.plugins.log.transitionchecker;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * Class to check whether a transition from one lifecycle transition to another
 * is valid
 * 
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Apr 6, 2009
 */
public class StandardLifecycleTransitionChecker implements LifecycleTransitionChecker {

	public Set<Integer> getAvailableLifecycleTransitions() {
		Set<Integer> result = new HashSet<Integer>();
		result.add(SCHEDULE);
		result.add(START);
		result.add(SUSPEND);
		result.add(RESUME);
		result.add(COMPLETE);
		return result;
	}

	public List<Integer> getAvailableTransitionsFrom(int currLifecycleTransition) {
		// return set of available transitions
		List<Integer> result = new LinkedList<Integer>();
		switch (currLifecycleTransition) {
			case SCHEDULE :
				result.add(START);
				break;
			case START :
				result.add(COMPLETE);
				result.add(SUSPEND);
				break;
			case SUSPEND :
				result.add(RESUME);
				break;
			case RESUME :
				result.add(SUSPEND);
				result.add(COMPLETE);
				break;
			default :
				result.add(UNKNOWN);
				break;
		}
		return result;
	}

	public boolean isValid(int fromLifecycleTransition, int toLifecycleTransition) {
		return getAvailableTransitionsFrom(fromLifecycleTransition).contains(toLifecycleTransition);
	}

	public int getIndexOfLifecycleTransitionString(String lifecycleTransition) {
		if (lifecycleTransition == null) {
			return UNKNOWN;
		} else {
			if (lifecycleTransition.equalsIgnoreCase("schedule")) {
				return SCHEDULE;
			} else if (lifecycleTransition.equalsIgnoreCase("start")) {
				return START;
			} else if (lifecycleTransition.equalsIgnoreCase("suspend")) {
				return SUSPEND;
			} else if (lifecycleTransition.equalsIgnoreCase("resume")) {
				return RESUME;
			} else if (lifecycleTransition.equalsIgnoreCase("complete")) {
				return COMPLETE;
			} else {
				return UNKNOWN;
			}
		}
	}

	public List<Integer> getAvailableTransitionsTo(int currLifecycleTransition) {
		List<Integer> result = new LinkedList<Integer>();
		switch (currLifecycleTransition) {
			case SCHEDULE :
				result.add(ALLLIFECYCLETRANSITION);
				break;
			case START :
				result.add(SCHEDULE);
				break;
			case SUSPEND :
				result.add(RESUME);
				result.add(START);
				break;
			case RESUME :
				result.add(SUSPEND);
				break;
			case COMPLETE :
				result.add(START);
				result.add(RESUME);
				break;
			default :
				result.add(UNKNOWN);
				break;
		}
		return result;
	}
}
