/**
 * 
 */
package org.processmining.models.fuzzyperformancediagram;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.SwingConstants;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.fuzzyperformancediagram.elements.FPDEdgeElement;
import org.processmining.models.fuzzyperformancediagram.elements.FPDNodeElement;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 19, 2009
 */
public class FPDImpl extends AbstractDirectedGraph<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>> implements
		FPD {

	protected final Set<FPDNode> nodes;
	protected final Set<FPDEdge<FPDNode, FPDNode>> arcs;

	public FPDImpl(String label) {
		super();
		nodes = new LinkedHashSet<FPDNode>();
		arcs = new LinkedHashSet<FPDEdge<FPDNode, FPDNode>>();
		getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.WEST);
		getAttributeMap().put(AttributeMap.LABEL, label);
	}

	@Override
	protected FPDImpl getEmptyClone() {
		return new FPDImpl(getLabel());
	}

	protected Map<? extends DirectedGraphElement, ? extends DirectedGraphElement> cloneFrom(
			DirectedGraph<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>> graph) {
		HashMap<DirectedGraphElement, DirectedGraphElement> mapping = new HashMap<DirectedGraphElement, DirectedGraphElement>();

		for (FPDNode a : graph.getNodes()) {
			mapping.put(a, addNode(a.getLabel()));
		}

		getAttributeMap().clear();
		AttributeMap map = graph.getAttributeMap();
		for (String key : map.keySet()) {
			getAttributeMap().put(key, map.get(key));
		}
		return mapping;
	}

	/**
	 * NODE
	 */
	public synchronized FPDNode addNode(String label) {
		FPDNode node = new FPDNodeElement(label, this);
		nodes.add(node);
		graphElementAdded(node);
		return node;
	}

	public synchronized FPDNode removeNode(FPDNode fpdNode) {
		removeSurroundingEdges(fpdNode);
		return removeNodeFromCollection(nodes, fpdNode);
	}

	public void removeNode(DirectedGraphNode cell) {
		if (cell instanceof FPDNode) {
			removeNode((FPDNode) cell);
		}
	}

	public synchronized Set<FPDNode> getNodes() {
		Set<FPDNode> newNodes = new HashSet<FPDNode>();
		newNodes.addAll(nodes);
		return newNodes;
	}

	public Set<FPDNode> getNodes(XEventClass eventClass) {
		Set<FPDNode> result = new HashSet<FPDNode>();
		for (FPDNode node : nodes) {
			if (node.hasEventClass(eventClass)) {
				result.add(node);
			}
		}
		return result;
	}

	public boolean addNodeMapping(FPDNode node, XEventClass eventClass) {
		if (nodes.contains(node)) {
			for (FPDNode originalnode : getNodes()) {
				if (originalnode.equals(node)) {
					originalnode.addNodeMapping(eventClass);
					return true;
				}
			}
		}
		return false;
	}

	public boolean removeNodeMapping(FPDNode node, XEventClass removedEventClass) {
		if (nodes.contains(node)) {
			for (FPDNode originalnode : getNodes()) {
				if (originalnode.equals(node)) {
					originalnode.removeNodeMapping(removedEventClass);
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * ARC
	 */
	public synchronized FPDEdge<FPDNode, FPDNode> addArc(FPDNode source, FPDNode target) {
		return addArc(source, target, null);
	}

	public synchronized FPDEdge<FPDNode, FPDNode> addArc(FPDNode source, FPDNode target, String label) {
		checkAddEdge(source, target);
		FPDEdgeElement a = new FPDEdgeElement(source, target);
		if (arcs.add(a)) {
			graphElementAdded(a);
			return a;
		} else {
			for (FPDEdge<FPDNode, FPDNode> existing : arcs) {
				if (existing.equals(a)) {
					if (label != null) {
						existing.getAttributeMap().put(AttributeMap.LABEL, label);
					}
					return existing;
				}
			}
		}
		assert (false);
		return null;
	}

	public synchronized FPDEdge<FPDNode, FPDNode> removeArc(FPDNode source, FPDNode target) {
		return removeFromEdges(source, target, arcs);
	}

	public FPDEdge<FPDNode, FPDNode> removeArc(FPDEdge<? extends FPDNode, ? extends FPDNode> edge) {
		return removeFromEdges(edge.getSource(), edge.getTarget(), arcs);
	}

	@SuppressWarnings("unchecked")
	public void removeEdge(DirectedGraphEdge edge) {
		if (edge instanceof FPDEdgeElement) {
			arcs.remove(edge);
		} else {
			assert (false);
		}
		graphElementRemoved(edge);
	}

	public synchronized FPDEdge<FPDNode, FPDNode> getArc(FPDNode source, FPDNode target) {
		Collection<FPDEdge<FPDNode, FPDNode>> set = getEdges(source, target, arcs);
		return (set.isEmpty() ? null : set.iterator().next());
	}

	public synchronized Set<FPDEdge<? extends FPDNode, ? extends FPDNode>> getEdges() {
		Set<FPDEdge<? extends FPDNode, ? extends FPDNode>> edges = new HashSet<FPDEdge<? extends FPDNode, ? extends FPDNode>>();
		edges.addAll(arcs);
		return edges;
	}

}